#define DEBUGPRINT

#include <Arduino.h>
#include <NZWorker.h>
#include <CustomFieldTable.h>
#include <NZNet.h>
#include <flash_hal.h>
#include <EEPROM.h>


#define PARAM_TABLE_SIZE    30
#define RH_ENABLE_EXPLICIT_RETRY_DEDUP 1

#define UPPER_ADDRESS       0
#define LORA_ADDRESS        1
#define NETWORK_ADDRESS     1

//#define SET_EEPROM
#define READ_EEPROM

NZWorker device(D8, D2);

void init2eeprom(uint8_t upper, uint8_t lora, uint16_t net, char* name, char* model){
    int mem = 0;
    EEPROM.begin(100);
    EEPROM.put(mem++, upper);
    EEPROM.put(mem++, lora);
    EEPROM.put(mem, net);
    mem+=2;

    for (int i = 0; i < 10; i++) EEPROM.put(mem++, name[i]);
    for (int i = 0; i < 10; i++) EEPROM.put(mem++, model[i]);

    EEPROM.commit();
    EEPROM.end();
}

void initFromEeprom(uint8_t& upper, uint8_t& lora, uint16_t& net, char* name, char* model){
    int mem = 0;
    EEPROM.begin(100);
    EEPROM.get(mem++, upper);
    EEPROM.get(mem++, lora);
    EEPROM.get(mem, net);
    mem += 2;
    
    for (int i = 0; i < 10; i++) EEPROM.get(mem++, name[i]);
    for (int i = 0; i < 10; i++) EEPROM.get(mem++, model[i]);

    EEPROM.end();
}

uint64_t measuretime;

void setup(){
    Serial.begin(115200);
    uint8_t upper, loraaddr;
    uint16_t netaddr;   
    char name[10] = "Device1";
    char model[10] = "Modeltest";

    #ifdef SET_EEPROM
        init2eeprom(UPPER_ADDRESS, LORA_ADDRESS, (uint16_t)NETWORK_ADDRESS, name, model);
    #endif

    initFromEeprom(upper, loraaddr, netaddr, name, model);
    Serial.printf("\n\n%d %d %d %s %s\n\n", upper, loraaddr, netaddr, name, model);

    Serial.printf("Device initialisation started %d, %d\n", PARAM_TABLE_SIZE, device.paramTable.maxSize);
    uint8_t initstatus = device.init(name, model, upper, loraaddr, netaddr);
    Serial.printf("Device initialisation completed, status %d\n", initstatus);
    measuretime = millis();
}

void loop(){
    device.listenMessages();
    if (millis() - measuretime > *(uint32_t*)(device.paramTable[MEASURE_TIMER_PARAM].mem)){
        measuretime = millis();
        device.emulateMeasure();
        device.telemetryTable.print();
    }
}
