#include <CustomField.h>

void CustomField::setData(uint8_t type, const char* name, void* mem){
    this->type = type;
    this->isActual = 1;

    if (name == NULL) this->name[0] = 0;
    else strncpy(this->name, name, MAX_NAME_SIZE);

    if (mem != NULL){
        this->mem = mem;
        return;
    }

    switch (type){
        case    INT8_TYPE   :   {this->mem = new uint8_t;  break;}
        case    INT16_TYPE  :   {this->mem = new uint16_t; break;}
        case    INT32_TYPE  :   {this->mem = new uint32_t; break;}
        case    INT64_TYPE  :   {this->mem = new uint64_t; break;}
        case    FLOAT32_TYPE:   {this->mem = new float_t ; break;}
        case    FLOAT64_TYPE:   {this->mem = new double_t; break;}
    };

}

uint8_t CustomField::valueSize(){
    switch (type){
        case    INT8_TYPE   :   return sizeof(uint8_t);
        case    INT16_TYPE  :   return sizeof(uint16_t);
        case    INT32_TYPE  :   return sizeof(uint32_t);
        case    INT64_TYPE  :   return sizeof(uint64_t);
        case    FLOAT32_TYPE:   return sizeof(float_t );
        case    FLOAT64_TYPE:   return sizeof(double_t);
        default             :   return 0;    
    };
}

uint8_t CustomField::nameSize(){
    return strlen(name) + 1;
}

uint8_t CustomField::bufferWriteValue(uint8_t* buf){
    memcpy((char*)buf, mem, valueSize());
    return (valueSize());
}

uint8_t CustomField::bufferWriteInfo(uint8_t* buf){
    uint8_t* pos = buf;
    *(pos++) = type;
    strcpy((char*)pos, name);
    return(nameSize() + 1);
}

void CustomField::updateValue(void* mem){
    memcpy(this->mem, mem, this->valueSize());
    isActual = 0;
}

void CustomField::print(){
    switch (type){
        case    INT8_TYPE   :   {Serial.printf("%10i", *(uint8_t*)mem);       break;}
        case    INT16_TYPE  :   {Serial.printf("%10i", *(uint16_t*)mem);      break;}
        case    INT32_TYPE  :   {Serial.printf("%10i", *(uint32_t*)mem);      break;}
        case    INT64_TYPE  :   {Serial.printf("%10lli", *(uint64_t*)mem);    break;}
        case    FLOAT32_TYPE:   {Serial.printf("%10f", *(float_t*)mem);       break;}
        case    FLOAT64_TYPE:   {Serial.printf("%10lf", *(double_t*)mem);     break;}
    };
}

void CustomParam::setData(uint8_t type, void* onChangeHandler, const char* name, void* mem){
    CustomField::setData(type, name, mem);
    this->onChangeHandler = onChangeHandler;
}
