#include <CustomFieldTable.h>

CustomFieldTable::CustomFieldTable(uint8_t maxSize){
    table = new CustomField[maxSize];
    this->maxSize = maxSize;
    size = 0;    
}

CustomField& CustomFieldTable::operator[](uint8_t pos){
    return table[pos];
}

uint8_t CustomFieldTable::indexOf(char* name){
    uint8_t result = -1;
    for (int i = 0; i < size; i++){
        if (strcmp(name, table[i].name) == 0) return i;
    }
    return result;
}

uint32_t CustomFieldTable::transmitedInfoSize(){
    uint8_t result = 0;
    for (int i = 0; i < size; i++) result += table[i].nameSize() + 1;
    return result;
}

uint32_t CustomFieldTable::transmitedDataSize(){
    uint8_t result = 0;
    for (int i = 0; i < size; i++) result += table[i].valueSize();
    return result;
}

uint8_t CustomFieldTable::bufferWriteData(uint8_t* buffer){
    uint8_t* pos = buffer;
    for (uint8_t i = 0; i < size; i++) pos += table[i].bufferWriteValue(pos);
    return pos - buffer;
}

uint8_t CustomFieldTable::bufferWriteInfo(uint8_t* buffer){
    uint8_t* pos = buffer;
    for (uint8_t i = 0; i < size; i++) pos += table[i].bufferWriteInfo(pos);
    return pos - buffer;
}

uint8_t CustomFieldTable::add(uint8_t type, char* name, void* mem){
    if (size == maxSize) return -1;
    table[size].setData(type, name, mem);
    size++;
    return 0;
}

void CustomFieldTable::print(){
    for (int i = 0; i < size; i++){
        Serial.printf("%30s %1d :  ", table[i].name, table[i].type);
        table[i].print();
        Serial.printf(" %1d\n", table[i].isActual);       
    }
}





CustomParamTable::CustomParamTable(uint8_t maxSize){
    table = new CustomParam[maxSize];
    this->maxSize = maxSize;
    size = 0;    
}

CustomParam& CustomParamTable::operator[](uint8_t pos){
    return table[pos];
}

uint8_t CustomParamTable::indexOf(char* name){
    uint8_t result = -1;
    for (int i = 0; i < size; i++){
        if (strcmp(name, table[i].name) == 0) return i;
    }
    return result;
}

uint32_t CustomParamTable::transmitedInfoSize(){
    uint8_t result = 0;
    for (int i = 0; i < size; i++) result += table[i].nameSize() + 1;
    return result;
}

uint32_t CustomParamTable::transmitedDataSize(){
    uint8_t result = 0;
    for (int i = 0; i < size; i++) result += table[i].valueSize();
    return result;
}

uint8_t CustomParamTable::add(uint8_t type, void* onChangeHandler, const char* name, void* mem){

    Serial.printf("Added field \'%s\' -> %p (%d) to ParamTable\n", name, name, type);

    if (size == maxSize) return -1;
    table[size].setData(type, onChangeHandler, name, mem);
    size++;
    return 0;
}

void CustomParamTable::print(){
    for (int i = 0; i < size; i++){
        Serial.printf("%30s %1d :  ", table[i].name, table[i].type);
        table[i].print();
        Serial.printf(" %1d, handler - %p\n", table[i].isActual, table[i].onChangeHandler);       
    }
}