#include <NZGateway.h>

#define     SSID "124"
#define     PASS "qwerqwer1"


NZGateway::NZGateway(uint8_t SSPin, uint8_t DIOpin) :   NZDevice(SSPin, DIOpin),
                                                        webServer(80),
                                                        messageManager(this)
{
    
}

uint8_t NZGateway::init(char* name, char* model){
    uint8_t result = NZDevice::init(name, model, 0, 0, 0);
    if (result) return result;
    if (initWiFi((char*)SSID, (char*)PASS)) return WIFI_ERROR;
    if (initWeb()) return WEB_ERROR;
    initServer();
    return 0;
}

uint8_t NZGateway::initWiFi(char* ssid, char* password){

    Serial.println("WiFi init started");
    uint8_t retries = 0;
    WiFi.disconnect();

    WiFi.begin(ssid, password);

    while ((WiFi.status() != WL_CONNECTED) & (retries < 100)) 
    {
        delay(500);
        Serial.print('.');
        retries++;
    }
    if (retries >= 100) return WIFI_ERROR;

    Serial.println();
    Serial.println(WiFi.localIP());

    return 0;
}

uint8_t NZGateway::initWeb(){

    char str[256];
    _jsonDoc.clear();

    _jsonDoc["name"] = this->deviceName;

    serializeJson(_jsonDoc, str);

    uint8_t respCode = sendJSONStringHTTP((uint8_t*)str, "http://192.168.1.100/lora/connect_gateway", "POST");
    if (respCode != 200) return WEB_ERROR;
    return 0;
}

void NZGateway::listenMessages(){
    bool recvStatus = 0;
    lora_recv_len = RH_MAX_MESSAGE_LEN;
  	if (router.available() && (recvStatus = router.recvfromAck(lora_recv_buf, &lora_recv_len))) {
        Serial.printf("GOT MESSAGE FROM %d, len: %d\n", router.headerAddr(), lora_recv_len);
        NZDevice::printMem(lora_recv_buf, lora_recv_len);

        httpClient.begin(wifiClient, "http://192.168.1.100/lora/send_command");
        httpClient.sendRequest("POST", lora_recv_buf, lora_recv_len);

        processMessage(router.headerAddr());
  	}
}

void NZGateway::processMessage(uint16_t from){
    messageManager.handle_rxMessage(from);
}


uint16_t NZGateway::sendRawStringHTTP(uint8_t* mem, const char* url, const char* method){
    httpClient.begin(wifiClient, url);
    Serial.printf("%s %s %s\n", mem, url, method);
    httpClient.addHeader("Content-type", "text/plain");
    uint16_t resCode = httpClient.sendRequest(method, mem);
    httpClient.end();
    return resCode; 
}

uint16_t NZGateway::sendJSONStringHTTP(uint8_t* mem, const char* url, const char* method){
    httpClient.begin(wifiClient, url);
    Serial.printf("%s %s %s\n", mem, url, method);
    httpClient.addHeader("Content-type", "application/json");
    uint16_t resCode = httpClient.sendRequest(method, mem);
    httpClient.end();
    return resCode; 
}

uint8_t NZGateway::initServer(){
    webServer.on("/set_param", HTTP_POST, std::bind(&NZGateway::set_param_handler, this));
    //webServer.on("/ping", HTTP_GET, std::bind(&NZGateway::ping_handler, this));    
    webServer.begin();
    return 0;
}

void NZGateway::set_param_handler(){
    // addr - 0, index - 1, type - 2

    _jsonDoc.clear();
    deserializeJson(_jsonDoc, webServer.arg("plain"));


    char mem[8];
    uint8_t len = 0;

    switch ((uint8_t)_jsonDoc["type"]){
        case INT8_TYPE: {*(uint8_t*)mem = (uint8_t)_jsonDoc["val"];     len = sizeof(uint8_t);  break;}
        case INT16_TYPE: {*(uint16_t*)mem = (uint16_t)_jsonDoc["val"];  len = sizeof(uint16_t);  break;}
        case INT32_TYPE: {*(uint32_t*)mem = (uint32_t)_jsonDoc["val"];  len = sizeof(uint32_t);  break;}
        case INT64_TYPE: {*(uint64_t*)mem = (uint64_t)_jsonDoc["val"];  len = sizeof(uint64_t);  break;}
        case FLOAT32_TYPE: {*(float_t*)mem = (float_t)_jsonDoc["val"];  len = sizeof(float_t);  break;}
        case FLOAT64_TYPE: {*(double_t*)mem = (double_t)_jsonDoc["val"];len = sizeof(double_t);  break;}     
    }

    lora_send_len = messageManager.set_param((uint8_t)_jsonDoc["index"], (void*)mem, len);

    router.sendtoDeviceWait(lora_send_buf, lora_send_len, (uint16_t)_jsonDoc["addr"]);

    //messageManager.setParameter((uint8_t)_jsonDoc[])

    webServer.send(200);
}



void NZGateway::listenWeb(){
    webServer.handleClient();
}

