#ifndef NZNET_H
#define NZNET_H

#define     BASE_URL    "http://192.168.1.100/lora/"

#include <CustomFieldTable.h>
#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>
#include <ESP8266WebServer.h>
#include <WiFiClient.h>
#include <ArduinoJson.h>
#include <NZRouter.h>

#define TRANSMITABLE_MESSAGE_LEN (RH_ROUTER_MAX_MESSAGE_LEN-1)
#define JSON_BUFFER_SIZE  1024

class NZGateway;
class NZDevice;

class NZNet {

public:

    typedef struct {
        uint8_t opCode;
        uint8_t data[TRANSMITABLE_MESSAGE_LEN];
    } Message;
    
    NZRouter* _router                                   = NULL;
    WiFiClient* _wifiClient                             = NULL;
    HTTPClient* _httpClient                             = NULL;
    CustomParamTable* _paramTable                       = NULL;
    CustomFieldTable* _telemetryTable                   = NULL; 
    
    NZNet(uint8_t* rxBuffer, uint8_t* txBuffer, NZRouter* router, WiFiClient* wifiClient = NULL, HTTPClient* httpClient = NULL);

    NZNet(NZGateway* gateway);

    NZNet(NZDevice* device);

    uint8_t ack();

    uint8_t set_param(uint8_t index, CustomParamTable& paramTable);

    uint8_t set_param(uint8_t index, void* mem, uint8_t size);

    uint8_t send_data(uint8_t index, CustomFieldTable& telemetryTable);

    uint8_t set_param_full(CustomParamTable& paramTable);

    uint8_t send_data_full(CustomFieldTable& telemetryTable);

    uint8_t server_register(char* name, char* model);

    uint8_t disconnect();


    void set_param_handler();

    void send_data_handler(uint16_t address);

    void set_param_full_handler();

    void send_data_full_handler(uint16_t address);

    void server_register_handler(uint16_t address);

    void disconnect_handler(uint16_t address);

    void handle_rxMessage(uint16_t address);

public:

    Message*  _txMessage;
    Message*  _rxMessage;
    StaticJsonDocument<JSON_BUFFER_SIZE> _jsonDoc;

};

#endif