#ifndef NZROUTER_H
#define NZROUTER_H

#include <RHReliableDatagram.h>
#include <RHGenericDriver.h>

#define DEBUGPRINT


#define ROUTING_TABLE_SIZE  30

// Error codes
#define RH_ROUTER_ERROR_NONE              0
#define RH_ROUTER_ERROR_INVALID_LENGTH    1
#define RH_ROUTER_ERROR_NO_ROUTE          2
#define RH_ROUTER_ERROR_TIMEOUT           3
#define RH_ROUTER_ERROR_NO_REPLY          4
#define RH_ROUTER_ERROR_UNABLE_TO_DELIVER 5

#define UPLINK_FLAG_MASK    1
#define READABLE_FLAG_MASK  2

#define RH_ROUTER_MAX_MESSAGE_LEN (RH_MAX_MESSAGE_LEN - sizeof(NZRouter::RoutedMessageHeader))
class NZRouter : public RHReliableDatagram {

public:

    typedef struct 
    {
        uint16_t addrFlags;
    } RoutedMessageHeader;
    

    typedef struct {
        RoutedMessageHeader header;
        uint8_t             data[RH_ROUTER_MAX_MESSAGE_LEN];
    } RoutedMessage;

    typedef enum {
        Inactive,
        Active,
        Unknown
    } RouteState;

    typedef struct {
        uint8_t destNetAddr;
        uint8_t next_hop_LoRa;
        uint8_t state;
        uint16_t last_timestamp;
    } RoutingTableEntry;

    NZRouter(RHGenericDriver& driver, uint8_t localGateway, uint8_t thisAddress = 0, uint16_t thisNetAddress = 0);

    NZRouter();

    void clearRoutingTable();

    bool init();

    void addRouteTo(uint8_t destNetAddr, uint8_t next_hop_LoRa, uint8_t state);

    void setThisNetworkAddress(uint16_t newNetAddr);

    void setThisLocalGateway(uint8_t newGateway);

    RoutingTableEntry* getRouteTo(uint16_t dest);

    void printRoutingTable();

    bool deleteRouteTo(uint16_t addr);
    
    void retireOldestRoute();

    uint8_t sendtoGatewayWait(uint8_t* buf, uint8_t len);

    uint8_t sendtoDeviceWait(uint8_t* buf, uint8_t len, uint16_t dest);

    uint8_t route(RoutedMessage* message, uint8_t messageLen);

    bool recvfromAck(uint8_t* buf, uint8_t* len, uint16_t* globalAddr = NULL, uint8_t* id = NULL);

    bool recvFromAckTimeout(uint8_t* buf, uint8_t* len, uint16_t timeout, uint16_t* globalAddr = NULL, uint8_t* id = NULL);

    uint16_t thisNetworkAddress();

    uint16_t headerAddr();

protected:

    void peekAtMessage(RoutedMessage* message, uint8_t messageLen);

    void deleteRoute(uint8_t index);

    void printBufferedMessage(uint8_t messageLen);


    uint8_t                 _thisNetworkAddress;
    uint8_t                 _thisGateway;
    uint8_t                 _lastE2ESequenceNumber;

private:

    static RoutedMessage    _tmpMessage;

    RoutingTableEntry       _routes[ROUTING_TABLE_SIZE];
    uint16_t                _routingTableUsage;



};
#endif
