#include <NZWorker.h>

#define W

NZWorker::NZWorker(uint8_t SSpin, uint8_t DIOPin) : NZDevice(SSpin, DIOPin){};

uint8_t NZWorker::initConnection(){

    uint8_t result = 0;

    //Отправка запроса на подключение

    
    lora_send_len = messageManager.server_register(deviceName, deviceModel);
    result = router.sendtoGatewayWait(lora_send_buf, lora_send_len);
    if (result) return CONNECTION_ERROR;
    
    //Ожидание подтверждения подключения
    int tm = millis();
    while ((millis() - tm < NETWORK_TIMEOUT) && !router.available() && !router.recvfromAck(lora_recv_buf, &(lora_recv_len = RH_MAX_MESSAGE_LEN))){
        //YIELD
    };
    
    //Если подтверждение не получено - ошибка
    if ((lora_recv_len == RH_MAX_MESSAGE_LEN) || (messageManager._rxMessage->opCode != 0)) return CONNECTION_ERROR;
    
    return 0;
}

uint8_t NZWorker::init(char* name, char* model, uint8_t upperAddress, uint8_t thisAddress, uint16_t thisNetAddress)
{
    uint8_t res = NZDevice::init(name, model, upperAddress, thisAddress, thisNetAddress);
    if (res) return res;
    initParameters();
    return initConnection();
}

void NZWorker::initParameters(){
    telemetryTable.add(FLOAT32_TYPE, "weather_temp");
    telemetryTable.add(FLOAT32_TYPE, "weather_pressure");
    telemetryTable.add(FLOAT32_TYPE, "weather_humidity");
    telemetryTable.print();
    paramTable.add(INT32_TYPE, NULL, "measure_timer_ms");
    *(uint32_t*)paramTable[MEASURE_TIMER_PARAM].mem = 60000;
}

void NZWorker::listenMessages(){
    bool recvStatus = 0;
    lora_recv_len = RH_MAX_MESSAGE_LEN;
  	if (router.available() && (recvStatus = router.recvfromAck(lora_recv_buf, &lora_recv_len))) {
        Serial.printf("GOT MESSAGE FROM %d, len: %d\n", router.headerAddr(), lora_recv_len);
        NZDevice::printMem(lora_recv_buf, lora_recv_len);

        processMessage(router.headerAddr());
  	}    
}

void NZWorker::processMessage(uint16_t from){
    messageManager.handle_rxMessage(from);
}

void NZWorker::emulateMeasure(){
    float_t val = random(2000, 2200) / 100.;
    *(float_t*)(telemetryTable[TEMP_PARAM].mem) = val;
    val = random(75000, 76000) / 100.;
    *(float_t*)(telemetryTable[PRESSURE_PARAM].mem) = val;
    val = random(5000, 7000) / 100.;
    *(float_t*)(telemetryTable[HUMIDITY_PARAM].mem) = val;

    lora_send_len = messageManager.send_data_full(telemetryTable);
    router.sendtoGatewayWait(lora_send_buf, lora_send_len);
}