#include <NZdevice.h>

uint8_t                 NZDevice::lora_recv_buf[RH_ROUTER_MAX_MESSAGE_LEN];
uint8_t                 NZDevice::lora_send_buf[RH_ROUTER_MAX_MESSAGE_LEN];

NZDevice::NZDevice(uint8_t SSpin, uint8_t DIOPin):
    driver(SSpin, DIOPin),
    router(driver, 0, 0, 0),
    paramTable(PARAM_TABLE_SIZE),
    telemetryTable(TELEMETRY_TABLE_SIZE),
    messageManager(this)
{
};

uint8_t NZDevice::init(char* name, char* model, uint8_t upperAddress, uint8_t thisAddress, uint16_t thisNetAddress){


    strcpy(this->deviceName, name);
    strcpy(this->deviceModel, model);
    
    #ifdef DEBUGPRINT
        Serial.print("Device init start\n");
    #endif
    

    if (initNetwork(upperAddress, thisAddress, thisNetAddress)) return NETWORK_INIT_ERROR;
    if (initDriver()) return DRIVER_INIT_ERROR;

    #ifdef DEBUGPRINT
        Serial.print("Current params table:\n");
        paramTable.print();
        Serial.printf("%d %d\n", router.thisAddress(), router.thisNetworkAddress());
    #endif

    return NONR_ERROR;
}

bool NZDevice::initNetwork(uint8_t upperAddress, uint8_t thisAddress, uint16_t thisNetAddress){
    bool result = 0;

    paramTable.add(INT8_TYPE, (void*)(&NZDevice::updateNetworkParams), "Upper_addr");
    paramTable.add(INT8_TYPE, (void*)(&NZDevice::updateNetworkParams), "LoRa_addr");
    paramTable.add(INT16_TYPE, (void*)(&NZDevice::updateNetworkParams), "Network_addr");  

    *(uint8_t*)(paramTable[UPPER_ADDRESS_PARAM].mem) = upperAddress;
    *(uint8_t*)(paramTable[LORA_ADDRESS_PARAM].mem) =      thisAddress;
    *(uint16_t*)(paramTable[NETWORK_ADDRESS_PARAM].mem) =  thisNetAddress;

    updateNetworkParams();
   
    result = !router.init();
    Serial.printf("Router init status: %d\n", result);
    return result;
}

bool NZDevice::initDriver(){
     
    #ifdef DEBUGPRINT
        Serial.print("Driver init start\n");
    #endif

    bool result = 0;

    paramTable.add(FLOAT32_TYPE, (void*)&NZDevice::initDriver, "Driver_frequency");
    paramTable.add(INT8_TYPE, (void*)&NZDevice::initDriver, "Driver_txpower");

    *(float*)(paramTable[DRIVER_FREQUENCY_PARAM].mem) = DRIVER_FREQUENCY_VAL;
    *(uint8_t*)(paramTable[DRIVER_TXPOWER_PARAM].mem) = DRIVER_TXPOWER_VAL;

    /*

    if (!paramTable.paramValue[DRIVER_FREQUENCY_PARAM].isActual) {

        result = addParam("Driver_frequency", FLOAT32_TYPE);
        result = addParam("Driver_txPower", INT8_TYPE);

    }

    *(float*)(paramTable.paramValue[DRIVER_FREQUENCY_PARAM].value) = DRIVER_FREQUENCY_VAL;
    *(uint8_t*)(paramTable.paramValue[DRIVER_TXPOWER_PARAM].value) = DRIVER_TXPOWER_VAL;

    */

    #ifdef DEBUGPRINT
        Serial.print("Parameter value updated\n");
    #endif

    driver.setFrequency(*(float*)(paramTable[DRIVER_FREQUENCY_PARAM].mem));
  	driver.setTxPower(*(uint8_t*)(paramTable[DRIVER_TXPOWER_PARAM].mem), false);

    return result;
};

bool NZDevice::listenMessages(){
    bool recvStatus = 0;
  	if (router.available() && (recvStatus = router.recvfromAck(lora_recv_buf, &lora_recv_len))) {

        Serial.printf("GOT MESSAGE FROM %d, len: %d\n", router.headerFrom(), lora_recv_len);
        printMem(lora_recv_buf, lora_recv_len);
  	}
    return recvStatus;
}

void NZDevice::updateNetworkParams(){
    router.setThisAddress(*(uint8_t*)paramTable[LORA_ADDRESS_PARAM].mem);
    router.setThisNetworkAddress(*(uint8_t*)paramTable[NETWORK_ADDRESS_PARAM].mem);
    router.setThisLocalGateway(*(uint8_t*)paramTable[UPPER_ADDRESS_PARAM].mem);
    Serial.printf("\n");
}

void NZDevice::updateDriverParams(){
    driver.setFrequency(*(float_t*)paramTable[DRIVER_FREQUENCY_PARAM].mem);
    driver.setTxPower(*(uint8_t*)paramTable[DRIVER_TXPOWER_PARAM].mem, false);
}

void NZDevice::printMem(void* mem, uint8_t size){
    for (int i = 0; i < size; i++) Serial.printf("%02x ", ((uint8_t*)mem)[i]);
    Serial.println();
}

void NZDevice::processMessage(uint16_t address){
    messageManager.handle_rxMessage(address);
}


