#ifndef NZDEVICE_H
#define NZDEVICE_H


#include <NZRouter.h>
#include <RH_RF95.h>
#include <NZNet.h>
#include <CustomFieldTable.h>


#define     BASE_API_URL http://192.168.1.100/lora/
#define     NETWORK_TIMEOUT         2500


#define     DRIVER_FREQUENCY_VAL    868.9    
#define     DRIVER_TXPOWER_VAL      13

//-------------------------------------------

#define     MESSAGE_BUFFER_SIZE     32
#define     PARAM_TABLE_SIZE       30
#define     TELEMETRY_TABLE_SIZE   30

#define     MAX_PARAM_NAME_LEN     30

#define     NONR_ERROR          0
#define     DRIVER_INIT_ERROR   1
#define     NETWORK_INIT_ERROR  2
#define     CONNECTION_ERROR    3
#define     WIFI_ERROR          4
#define     WEB_ERROR           5


#define     UPPER_ADDRESS_PARAM     0 
#define     LORA_ADDRESS_PARAM      1
#define     NETWORK_ADDRESS_PARAM   2

#define     DRIVER_FREQUENCY_PARAM  3
#define     DRIVER_TXPOWER_PARAM    4



class NZDevice {
friend class NZNet;
public:
    
    
    NZDevice( uint8_t SSPin, uint8_t DIOpin);

    uint8_t init(char* name, char* model, uint8_t upperAddress, uint8_t thisAddress, uint16_t thisNetAddress);

    bool listenMessages();

    NZRouter router;
    RH_RF95 driver;

    CustomParamTable paramTable;
    CustomFieldTable telemetryTable;
    
protected:

    void printMem(void* mem, uint8_t size);

    void updateNetworkParams();

    void updateDriverParams();

    bool initDriver();

    bool initNetwork(uint8_t upperAddress, uint8_t thisAddress = 0, uint16_t thisNetAddress = 0);

    bool connect();

    void processMessage(uint16_t addr);

    void reboot();

    static uint8_t  lora_recv_buf[RH_ROUTER_MAX_MESSAGE_LEN];
    uint8_t         lora_recv_len = RH_ROUTER_MAX_MESSAGE_LEN;

    static uint8_t  lora_send_buf[RH_ROUTER_MAX_MESSAGE_LEN];
    uint8_t         lora_send_len = RH_ROUTER_MAX_MESSAGE_LEN;

    NZNet           messageManager;

    char deviceName[40];
    char deviceModel[40];

};

#endif